﻿-- Scaling Alert
-- Created by: @Enodoc, UESP
-- Based on Cyrodiil Alert 2

SA = {}

SA.name = "ScalingAlert"
SA.version = "0.∞ (Final)"
SA.initialised = false

-- Default settings.
SA.defaults = {
	locx            	= 535,
	locy            	= 150,
	scaling         	= 1.0,
	notifyDelay		= 5,
	chatOutput		= true,
	onlyChat		= false,
	showScaling		= true,
	showMode		= true,
	showModeChange		= true,
	showBattle		= true,
	showBattleAlways	= true,
--	showInit		= true,
	locked			= true,
	chooseUI		= "ESO UI",
	sound			= true,
	textAlign       	= "CENTER",
}

SA_ACE = LibStub("AceTimer-3.0")

function SA.Initialise(eventCode, addOnName)

	-- Initialize self only.
	if (SA.name ~= addOnName) then return end

	-- Load saved variables.
	SA.vars = ZO_SavedVars:NewAccountWide("SA_SavedVariables", 1, nil, SA.defaults)

	
	ScalingAlert:ClearAnchors();
	ScalingAlert:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, SA.vars.locx, SA.vars.locy)

	ScalingAlertBG:SetAlpha(0)
    
	ScalingAlertNotify1:ClearAnchors();
	ScalingAlertNotify1:SetAnchor(TOP, ScalingAlert, TOP, 0, 0)
	ScalingAlertNotify1:SetWidth( 800 )
	ScalingAlertNotify1:SetHeight( 40 )

	ScalingAlertNotify2:ClearAnchors();
	ScalingAlertNotify2:SetAnchor(TOP, ScalingAlertNotify1, BOTTOM, 0, 0)
	ScalingAlertNotify2:SetWidth( 800 )
	ScalingAlertNotify2:SetHeight(40 )

	ScalingAlertNotify3:ClearAnchors();
	ScalingAlertNotify3:SetAnchor(TOP, ScalingAlertNotify2, BOTTOM, 0, 0)
	ScalingAlertNotify3:SetWidth( 800 )
	ScalingAlertNotify3:SetHeight( 70 )

        
    -- Event registration.    
	EVENT_MANAGER:RegisterForEvent("SA", EVENT_PLAYER_ACTIVATED, SA.OnUILoad)
	EVENT_MANAGER:RegisterForEvent("SA", EVENT_ACTION_LAYER_POPPED, SA.ShowInterface)
	EVENT_MANAGER:RegisterForEvent("SA", EVENT_ACTION_LAYER_PUSHED, SA.HideInterface)
	EVENT_MANAGER:RegisterForEvent("SA", EVENT_VETERAN_DIFFICULTY_CHANGED, SA.DungeonMode)
	EVENT_MANAGER:RegisterForEvent("SA", EVENT_GROUP_VETERAN_DIFFICULTY_CHANGED, SA.DungeonModeGroup)

	-- colour setup
	SA.col0 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_CON_COLORS,CON_TRIVIAL)
	SA.col1 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_CON_COLORS,CON_EASY)
	SA.col2 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_CON_COLORS,CON_APPROPRIATE)
	SA.col3 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_CON_COLORS,CON_DIFFICULT)
	SA.col4 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_CON_COLORS,CON_IMPOSSIBLE)
	SA.colW = ZO_ColorDef:New(0.9,0.9,0.9,1)
	SA.colQ0 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_ITEM_QUALITY_COLORS,ITEM_QUALITY_TRASH)
	SA.colQ1 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_ITEM_QUALITY_COLORS,ITEM_QUALITY_NORMAL)
	SA.colQ2 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_ITEM_QUALITY_COLORS,ITEM_QUALITY_MAGIC)
	SA.colQ3 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_ITEM_QUALITY_COLORS,ITEM_QUALITY_ARCANE)
	SA.colQ4 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_ITEM_QUALITY_COLORS,ITEM_QUALITY_ARTIFACT)
	SA.colQ5 = ZO_ColorDef:FromInterfaceColor(INTERFACE_COLOR_TYPE_ITEM_QUALITY_COLORS,ITEM_QUALITY_LEGENDARY)
	
	-- con names
	SA.con0 = "Trivial"
	SA.con1 = "Easy"
	SA.con2 = "Appropriate"
	SA.con3 = "Difficult"
	SA.con4 = "Impossible"


    -- Configuration Menu setup.
    SA.CreateConfigMenu()
    SA.obsoleteNotice = false
    SA.initialised = true    



end


EVENT_MANAGER:RegisterForEvent("SA", EVENT_ADD_ON_LOADED, SA.Initialise)


function SA.HideInterface(eventCode, layerIndex, activeLayerIndex)
	if (activeLayerIndex == 3) then
		ScalingAlert:SetHidden(true)
	end
end

function SA.ShowInterface(...)
	ScalingAlert:SetHidden(false)
end

function SA.Obsolete()
	SA.vars.chooseUI = "ESO UI"
	SA.vars.notifyDelay = 8
	if not SA.obsoleteNotice then
		outputString1=SA.col2:Colorize("ScalingAlert") .. " is now deprecated"
		outputString2="Due to the release of One Tamriel, " .. SA.col2:Colorize("ScalingAlert") .. " has become obsolete, and will no longer be supported. Please uninstall this add-on, and thanks for testing!"
		d(outputString1)
		d(outputString2)
		SA.NotifyMessage1(outputString1,outputString2)
		SA.timerId1 = SA_ACE:ScheduleTimer("ClearNotify1", SA.vars.notifyDelay)
		SA.timerId2 = SA_ACE:ScheduleTimer("ClearNotify2", SA.vars.notifyDelay)
		SA.timerId3 = SA_ACE:ScheduleTimer("ClearNotify3", SA.vars.notifyDelay)
		SA.obsoleteNotice=true
	end
end

function SA.Scaling()
	local zone = GetUnitZone("player")
	if (zone == SA.zone) then
		do return end   -- don't reshow for same zone
	end
	SA.zone = zone
	local zoneIndex = GetUnitZoneIndex("player")
	local scaling = DoesCurrentZoneAllowScalingByLevel()
	-- SCALE_LEVEL_CONSTRAINT (scaleConstraint) is
	-- 0: *_NORMAL_MIN_NORMAL_MAX
	-- 1: *_NORMAL_MIN_CHAMPION_RANK_MAX
	-- 2: *_CHAMPION_RANK_MIN_CHAMPION_RANK_MAX
	local scaleConstraint,minLevel,maxLevel = GetCurrentZoneLevelScalingConstraints()
	if (scaleConstraint == 2) then
		minLevel = minLevel+50
	end
	if (scaleConstraint ~= 0 and maxLevel ~= 0) then
		maxLevel = maxLevel+50
	end
	local battle = DoesCurrentZoneAllowBattleLevelScaling()
	local battlePlayer = IsUnitBattleLeveled("player")
	local battleChamp = IsUnitChampionBattleLeveled("player")
	local level = GetUnitLevel("player")
	local rank = GetUnitChampionPoints("player")
	local effective = GetUnitEffectiveLevel("player")
	local battleLevel = GetUnitBattleLevel("player")
	local battleRank = GetUnitChampionBattleLevel("player")
	local mode = IsUnitUsingVeteranDifficulty("player")
	local group = IsUnitGrouped("player")
	local leader = GetGroupLeaderUnitTag()
	local leaderLevel = GetUnitLevel(leader)
	local leaderRank = GetUnitChampionPoints(leader)
	local leaderMode = IsUnitUsingVeteranDifficulty(leader)
	local championCap = GetChampionPointsPlayerProgressionCap()
	local scaledLevel
	local scaledRank
	local dungeonMode
	if not group then
		scaledLevel = level
		scaledRank = rank
		dungeonMode = mode
	else
		scaledLevel = leaderLevel
		scaledRank = leaderRank
		dungeonMode = leaderMode
	end
	local battleCol
	if not battlePlayer then
		battleCol = SA.col0
	else
		battleCol = SA.col3
	end
	local poiZone, poiIndex = GetCurrentSubZonePOIIndices()
	local dungeon = IsPOIGroupDungeon(poiZone, poiIndex)

	if ((battle) and (not battlePlayer) and (not SA.vars.showBattleAlways)) then
--	d("SA Debug: battle level zone, not battle level player, not showing zones")
		battle = false
	end

-- zone names, by Votan
	local i = 1
	local zones = {}
	local zbn = GetZoneNameByIndex
	local zoneName
	while true do
		zoneName = zbn(i)
		if zoneName == "" then break end
		zones[zoneName] = i
		i = i + 1
	end

	local minLevels = {
		[zones["Stros M'Kai"]]		= 4,
		[zones["Betnikh"]]		= 5,
		[zones["Glenumbra"]]		= 3,
		[zones["Stormhaven"]]		= 16,
		[zones["Rivenspire"]]		= 24,
		[zones["Alik'r Desert"]]	= 31,
		[zones["Bangkorai"]]		= 37,
		[zones["Khenarthi's Roost"]]	= 4,
		[zones["Auridon"]]		= 3,
		[zones["Grahtwood"]]		= 16,
		[zones["Greenshade"]]		= 24,
		[zones["Malabal Tor"]]		= 31,
		[zones["Reaper's March"]]	= 37,
		[zones["Bleakrock Isle"]]	= 4,
		[zones["Bal Foyen"]]		= 5,
		[zones["Stonefalls"]]		= 3,
		[zones["Deshaan"]]		= 15,
		[zones["Shadowfen"]]		= 24,
		[zones["Eastmarch"]]		= 31,
		[zones["The Rift"]]		= 37,
		[zones["Coldharbour"]]		= 44,
		[zones["Craglorn"]]		= 51,
		[zones["Cyrodiil"]]		= 10,
		[zones["Wrothgar"]]		= 3,
	}
	local maxLevels = {
		[zones["Stros M'Kai"]]		= 5,
		[zones["Betnikh"]]		= 6,
		[zones["Glenumbra"]]		= 15,
		[zones["Stormhaven"]]		= 23,
		[zones["Rivenspire"]]		= 31,
		[zones["Alik'r Desert"]]	= 37,
		[zones["Bangkorai"]]		= 43,
		[zones["Khenarthi's Roost"]]	= 6,
		[zones["Auridon"]]		= 15,
		[zones["Grahtwood"]]		= 23,
		[zones["Greenshade"]]		= 31,
		[zones["Malabal Tor"]]		= 37,
		[zones["Reaper's March"]]	= 43,
		[zones["Bleakrock Isle"]]	= 5,
		[zones["Bal Foyen"]]		= 6,
		[zones["Stonefalls"]]		= 15,
		[zones["Deshaan"]]		= 24,
		[zones["Shadowfen"]]		= 31,
		[zones["Eastmarch"]]		= 37,
		[zones["The Rift"]]		= 43,
		[zones["Coldharbour"]]		= 50,
		[zones["Craglorn"]]		= 64,
		[zones["Cyrodiil"]]		= 66,
		[zones["Wrothgar"]]		= 66,
	}
--	minLevel=minLevels[zoneIndex]
--	maxLevel=maxLevels[zoneIndex]

	local con = GetCon(minLevel, level)
	if (dungeon or zone=="Maelstrom Arena" or zone=="Dragonstar Arena") and dungeonMode then
		con = GetCon(51, level)
	end
	local conCol
	if con < CON_APPROPRIATE then
		conCol = SA.col1
	elseif con > CON_APPROPRIATE then
		conCol = SA.col4
	else
		conCol = SA.col2
	end	

	if (SA.timerId1) then
		SA_ACE:CancelTimer(SA.timerId1)
	end
	if (SA.timerId2) then
		SA_ACE:CancelTimer(SA.timerId2)
	end


	local strLevel
	local strScaled
	local strBattle
	local strCol = SA.col2
	if battle and scaled then 	--force "battle-scaled" (ie, Group Finder Dungeon) to show CP 160
		scaledLevel = 50
		scaledRank = 160
	end
	if level < 50 then
		strLevel = zo_strformat("<<1>>", level)
	else
		strLevel = zo_strformat("|t24:24:<<X:1>>|t<<2>>", GetVeteranRankIcon(), rank)
	end
	if maxLevel ~= 0 and scaledLevel < minLevel then
		scaledLevel = minLevel
		strCol = SA.col4
	end
	if scaledRank > championCap then
		scaledRank = championCap
		strCol = SA.col1
	end
	if scaledLevel < 50 then
		strScaled = zo_strformat("<<1>>", scaledLevel)
	else
		strScaled = zo_strformat("|t24:24:<<X:1>>|t<<2>>", GetVeteranRankIcon(), scaledRank)
	end
	if ((battleLevel < 50) and battlePlayer) then
		strBattle = zo_strformat("<<1>>", battleLevel)
	elseif ((battleLevel < 50) and (not battlePlayer)) then
		strBattle = zo_strformat("(N/A)")
	else
		strBattle = zo_strformat("|t24:24:<<X:1>>|t<<2>>", GetVeteranRankIcon(), battleRank)
	end

	local outputString1
	local outputString2
	local outputString3

	if scaling and battle and (SA.vars.showScaling or SA.vars.showBattle) then
		outputString1 = SA.colW:Colorize("Entered ") .. SA.col3:Colorize("Battle-Leveled ") .. SA.col2:Colorize("Scaled Instance") .. SA.colW:Colorize(": " .. zone)
		if maxLevel ~= 0 and (dungeon or zone=="Maelstrom Arena" or zone=="Dragonstar Arena") and dungeonMode then
			outputString2 = SA.colW:Colorize("Min Level: ") .. conCol:Colorize(zo_strformat("|t24:24:<<X:1>>|t<<2>>", GetVeteranRankIcon(), "1")) .. SA.colW:Colorize(", Player Level: " .. strLevel) .. SA.colW:Colorize(", Battle Level: ") .. battleCol:Colorize(strBattle) .. SA.colW:Colorize(", Enemy Level: ") .. strCol:Colorize(strScaled)
		elseif maxLevel ~= 0 and minLevel < 50 then
			outputString2 = SA.colW:Colorize("Min Level: ") .. conCol:Colorize(minLevel) .. SA.colW:Colorize(", Player Level: " .. strLevel) .. SA.colW:Colorize(", Battle Level: ") .. battleCol:Colorize(strBattle) .. SA.colW:Colorize(", Enemy Level: ") .. strCol:Colorize(strScaled)
		elseif maxLevel ~= 0 and minLevel > 50 then
			outputString2 = SA.colW:Colorize("Min Level: ") .. conCol:Colorize(zo_strformat("|t24:24:<<X:1>>|t<<2>>", GetVeteranRankIcon(), minLevel-50)) .. SA.colW:Colorize(", Player Level: " .. strLevel) .. SA.colW:Colorize(", Battle Level: ") .. battleCol:Colorize(strBattle) .. SA.colW:Colorize(", Enemy Level: ") .. strCol:Colorize(strScaled)
		else
			outputString2 = SA.colW:Colorize("Player Level: " .. strLevel) .. SA.colW:Colorize(", Battle Level: ") .. battleCol:Colorize(strBattle) .. SA.colW:Colorize(", Enemy Level: ") .. strCol:Colorize(strScaled)
		end
	elseif scaling and SA.vars.showScaling then
		outputString1 = SA.colW:Colorize("Entered ") .. SA.col2:Colorize("Scaled Instance") .. SA.colW:Colorize(": " .. zone)
		if maxLevel ~= 0 and (dungeon or zone=="Maelstrom Arena" or zone=="Dragonstar Arena") and dungeonMode then
			outputString2 = SA.colW:Colorize("Min Level: ") .. conCol:Colorize(zo_strformat("|t24:24:<<X:1>>|t<<2>>", GetVeteranRankIcon(), "1")) .. SA.colW:Colorize(", Player Level: " .. strLevel) .. SA.colW:Colorize(", Enemy Level: ") .. strCol:Colorize(strScaled)
		elseif maxLevel ~= 0 and minLevel < 50 then
			outputString2 = SA.colW:Colorize("Min Level: ") .. conCol:Colorize(minLevel) .. SA.colW:Colorize(", Player Level: " .. strLevel) .. SA.colW:Colorize(", Enemy Level: ") .. strCol:Colorize(strScaled)
		elseif maxLevel ~= 0 and minLevel > 50 then
			outputString2 = SA.colW:Colorize("Min Level: ") .. conCol:Colorize(zo_strformat("|t24:24:<<X:1>>|t<<2>>", GetVeteranRankIcon(), minLevel-50)) .. SA.colW:Colorize(", Player Level: " .. strLevel) .. SA.colW:Colorize(", Enemy Level: ") .. strCol:Colorize(strScaled)
		else
			outputString2 = SA.colW:Colorize("Player Level: " .. strLevel) .. SA.colW:Colorize(", Enemy Level: ") .. strCol:Colorize(strScaled)
		end
	elseif battle and SA.vars.showBattle then
		outputString1 = SA.colW:Colorize("Entered ") .. SA.col3:Colorize("Battle Level Zone") .. SA.colW:Colorize(": " .. zone)
		outputString2 = SA.colW:Colorize("Player Level: " .. strLevel) .. SA.colW:Colorize(", Battle Level: ") .. battleCol:Colorize(strBattle)
	end
	if (dungeon or zone=="Maelstrom Arena" or zone=="Dragonstar Arena") and dungeonMode and SA.vars.showMode then
		outputString3 = SA.colW:Colorize("Dungeon Mode: ") .. SA.colQ4:Colorize("Veteran")
	elseif (dungeon or zone=="Maelstrom Arena" or zone=="Dragonstar Arena") and not dungeonMode and SA.vars.showMode then
		outputString3 = SA.colW:Colorize("Dungeon Mode: ") .. SA.colQ3:Colorize("Normal")
	end
	


	if (SA.vars.chatOutput) then
		if (scaling and SA.vars.showScaling) or (battle and SA.vars.showBattle) then
			d(outputString1)
			d(outputString2)
		end
		if (dungeon or zone=="Maelstrom Arena" or zone=="Dragonstar Arena") and SA.vars.showMode then
			d(outputString3)
		end
	end
	
	if (not SA.vars.onlyChat) then
		if (scaling and SA.vars.showScaling) or (battle and SA.vars.showBattle) then
			SA.NotifyMessage1(outputString1,outputString2)
		end
		if (dungeon or zone=="Maelstrom Arena" or zone=="Dragonstar Arena") and SA.vars.showMode then
			SA.NotifyMessage3(outputString3)
		end

		SA.timerId1 = SA_ACE:ScheduleTimer("ClearNotify1", SA.vars.notifyDelay)
		SA.timerId2 = SA_ACE:ScheduleTimer("ClearNotify2", SA.vars.notifyDelay)
		SA.timerId3 = SA_ACE:ScheduleTimer("ClearNotify3", SA.vars.notifyDelay)
	end
end

function SA.OnUILoad(eventCode)
--	d("SA Debug: UI Loaded, Player Activated")
--	zo_callLater(SA.Scaling,200)
	zo_callLater(SA.Obsolete,200)
end

function SA.DungeonMode(eventCode,unitTag,isDifficult)
	local unitName = GetUnitName(unitTag)
	local outputString
	if isDifficult and SA.vars.showMode and SA.vars.showModeChange then
		outputString = SA.col2:Colorize(unitName) .. SA.colW:Colorize(" changed Dungeon Mode to ") .. SA.colQ4:Colorize("Veteran")
	elseif not isDifficult and SA.vars.showMode and SA.vars.showModeChange then
		outputString = SA.col2:Colorize(unitName) .. SA.colW:Colorize(" changed Dungeon Mode to ") .. SA.colQ3:Colorize("Normal")
	end
	
	if (SA.vars.chatOutput) then
		d(outputString)
	end
	
	if (not SA.vars.onlyChat) then
		SA.NotifyMessage3(outputString)
		SA.timerId3 = SA_ACE:ScheduleTimer("ClearNotify3", SA.vars.notifyDelay)
	end
end

function SA.DungeonModeGroup(eventCode,isVeteranDifficulty)
	local unitTag = GetGroupLeaderUnitTag()
	local unitName = GetUnitName(unitTag)
	local outputString
	if isVeteranDifficulty and SA.vars.showMode and SA.vars.showModeChange then
		outputString = SA.col2:Colorize(unitName) .. SA.colW:Colorize(" changed Dungeon Mode to ") .. SA.colQ4:Colorize("Veteran")
	elseif not isVeteranDifficulty and SA.vars.showMode and SA.vars.showModeChange then
		outputString = SA.col2:Colorize(unitName) .. SA.colW:Colorize(" changed Dungeon Mode to ") .. SA.colQ3:Colorize("Normal")
	end

	if (SA.vars.chatOutput) then
		d(outputString)
	end
	
	if (not SA.vars.onlyChat) then
		SA.NotifyMessage3(outputString)
		SA.timerId3 = SA_ACE:ScheduleTimer("ClearNotify3", SA.vars.notifyDelay)
	end
end

function SA_ACE:ClearNotify1()
	ScalingAlertNotify1:SetText("")
end

function SA_ACE:ClearNotify2()
	ScalingAlertNotify2:SetText("")
end

function SA_ACE:ClearNotify3()
	ScalingAlertNotify3:SetText("")
end

function SA.NotifyMessage1(msg1,msg2)
	if not SA.vars.sound then sound = nil end
	if SA.vars.chooseUI == "ESO UI" then
		CENTER_SCREEN_ANNOUNCE:AddMessage(0, CSA_EVENT_COMBINED_TEXT, "Champion_PointsCommitted", msg1, msg2, nil, nil, nil, nil, SA.vars.notifyDelay*1000, nil)
	else
		ScalingAlertNotify1:SetText(msg1)
		ScalingAlertNotify2:SetText(msg2)
	end
end
function SA.NotifyMessage3(message)
	if not SA.vars.sound then sound = nil end
	if SA.vars.chooseUI == "ESO UI" then
		CENTER_SCREEN_ANNOUNCE:AddMessage(0, CSA_EVENT_SMALL_TEXT, "Champion_CycledToMage", message, nil, nil, nil, nil, nil, SA.vars.notifyDelay*1000, nil)
	else
		ScalingAlertNotify3:SetText(message)
	end
end

function SA.Movable()
	SA.vars.locked = not SA.vars.locked
	ScalingAlert:SetMovable(not SA.vars.locked)
	ScalingAlert:SetMouseEnabled(not SA.vars.locked)
	if (not SA.vars.locked) then
		ScalingAlertBG:SetAlpha(0.5)
	else
		ScalingAlertBG:SetAlpha(0)
	end
	if (SA.vars.locked) then
		SA.vars.locx = math.floor(ScalingAlert:GetLeft())
		SA.vars.locy = math.floor(ScalingAlert:GetTop())
	end
end

function SA.MoveStart()
    ScalingAlertBG:SetAlpha(0.5)
end 

function SA.MoveStop()
--    ScalingAlertBG:SetAlpha(0)
end 

function SA.SetTextAlign(newValue)
    SA.vars.textAlign = newValue
    local alignText = {}
    alignText["LEFT"]   = 0
    alignText["CENTER"] = 1
    alignText["RIGHT"]  = 2
    ScalingAlertNotify1:SetHorizontalAlignment(alignText[newValue])
end

-- SLASH COMMAND FUNCTIONALITY
function SAslash(text )
	if (text == "show") then
		return SA.Movable()
	end
	if (text == "hide") then
		return SA.Movable()
	end
	if (text == "zone") then
		SA.zone = nil
		return SA.Scaling()
	end
	if (text == "clear") then
		SA_ACE:ClearNotify1()
		SA_ACE:ClearNotify2()
		SA_ACE:ClearNotify3()
	end
	if (text == "help") then
		d("Avalable slash commands: show, hide, zone, help")
	end
end -- SAslash
 
SLASH_COMMANDS["/sa"] = SAslash